/**
 * Facebook アクセス用クラス
 * @constructor
 */
AccessFacebook = function () {
};

AccessFacebook.prototype = function () {

  /**
   * Facebook URLs
   */
  var URLS = {
    TOP: 'https://www.facebook.com/',
    DIALOG_OAUTH: 'https://www.facebook.com/dialog/oauth',
    DIALOG_FEED: 'https://www.facebook.com/dialog/feed',
    GRAPH_ME: 'https://graph.facebook.com/me',
  };
  
  /**
   * アクセストークン保存(Local Storage)のためのキー
   * @type {String}
   */
  var KEY_ACCESS_KEY = 'facebook_access_key';
  
  /**
   * Facebook API にアクセスする際のオプション情報
   * @type {Object.<string, string>}
   */
  var options;
  
  // ===== 以下、Private 関数群 =====
  /**
   * アクセスキーを保存
   */
  function saveAccessKey(obj) {
    if (obj == null) {
      localStorage.removeItem(KEY_ACCESS_KEY);
    }
    else if (typeof obj === 'object') {
      localStorage.setItem(KEY_ACCESS_KEY, JSON.stringify(obj));
    }
  }
  
  /**
   * アクセスキーの取得
   */
  function loadAccessKey() {
    return JSON.parse(localStorage.getItem(KEY_ACCESS_KEY))||{accessToken: '',userId: ''};
  }
  
  /**
   * アクセストークン確認
   */
  function hasAccessTokenInner() {
    return isNotEmpty(options.accessToken) && isNotEmpty(options.userId);
  }
  
  /**
   * アクセストークンの破棄
   */
  function clearAccessTokenInner() {
    debugLog('clear stored access key');
    saveAccessKey(null);
    options.accessToken = null;
    options.userId = null;
  }

  /**
   * Facebook ログイン実処理
   * ログインダイアログを表示し、アカウントの基本情報を取得する
   */
  function doLogin(isForce,callback) {
    debugLog('login start...');
    
    if (!isForce) {
      // アクセストークン取得済みの場合
      if (hasAccessTokenInner()) {
        getBasicInfo(true);
        return;
      }
    }

    // Facebook: Dialog OAuth
    var url = URLS.DIALOG_OAUTH;
    url = appendURLParam(url, 'client_id', options.clientId);
    url = appendURLParam(url, 'redirect_uri', options.redirectUrlWhenLogin);
    url += '&response_type=token&scope=user_about_me,publish_actions';
    
    var wnd = window.open(url, '_blank', 'location=yes');
    wnd.addEventListener('loaderror', function (evt) {
        debugLog('window loaderror');
        debugLog(evt);
        
        wnd.close();
    });

    wnd.addEventListener('loadstart', function (evt) {
        debugLog('window loadstart');
        debugLog(evt);

        if (evt.url.indexOf(options.redirectUrlWhenLogin) !== 0) {
          debugLog('event.url != options.redirectUrlWhenLogin');
          // options.redirectUrl から始まる URL 以外の場合はその画面を表示するので処理しない
          return;
        }

        wnd.close();
        options.accessToken = evt.url.substr(evt.url.indexOf('#') + 1);
        getBasicInfo(false,callback);
    });
  }

  /**
   * Facebook ログアウト実処理
   */
  function doLogout() {
    debugLog('logout start...');
    
    // Facebookページを開く
    var url = URLS.TOP;
    var wnd = window.open(url, '_blank', 'location=yes');
    wnd.addEventListener('loaderror', function (evt) {
        debugLog('window loaderror');
        debugLog(evt);
    });
    wnd.addEventListener('loadstart', function(evt) {
        debugLog('window loadstart');
        debugLog(evt);
        
        if (evt.url.indexOf('logout.php') !== -1) {
          clearAccessTokenInner();
          wnd.close();
        }
    });
  }

  /**
   * Facebook フィード投稿処理
   */
  function doFeed(linkUrl, name, caption) {
    debugLog('feed start...');
    
    // Facebook: Dialog Feed
    // ※nameとcaptionは非推奨なパラメータのため、渡しているけど反映はされない模様
    var url = URLS.DIALOG_FEED;
    url = appendURLParam(url, 'app_id', options.clientId);
    url = appendURLParam(url, 'link', linkUrl);
    url = appendURLParam(url, 'name', name);
    url = appendURLParam(url, 'caption', caption);
    url = appendURLParam(url, 'redirect_uri', options.redirectUrlWhenFeed);

    var wnd = window.open(url, '_blank', 'location=yes');

    wnd.addEventListener('loaderror', function (evt) {
        debugLog('window loaderror');
        debugLog(evt);
        
        wnd.close();
    });
  
    wnd.addEventListener('loadstart', function (evt) {
        debugLog('window loadstart');
        debugLog(evt);

        if (evt.url.indexOf(options.redirectUrlWhenFeed) !== 0) {
          debugLog('event.url != options.redirectUrlWhenLogin');
          // options.redirectUrl から始まる URL 以外の場合はその画面を表示するので処理しない
          return;
        }

        wnd.close();
    });
  }
  
  /**
   * アカウントの基本情報を取得する
   * ※ isDoLogin が true の場合、情報取得に成功するとニュースフィード画面を表示する
   * ※ isDoLogin が true の場合、情報取得に失敗するとログイン処理に遷移する
   * @param {boolean} isDoLogin ログイン操作からの呼び出しの場合 true
   */
  function getBasicInfo(isDoLogin,callback) {
    debugLog('getBasicInfo start...');
    
    // Facebook: Graph Me
    var url = URLS.GRAPH_ME + '?' + options.accessToken;

    $.ajax({
      url: url,
      type: 'GET'
    }).done(function (data) {
        debugLog('success /me');
        debugLog(data);
        
        // 情報保存
        options.userId = data.id;
        saveAccessKey({
          accessToken: options.accessToken,
          userId: options.userId
        });

        if (isDoLogin) {
          showNewsFeed();
        }
        callback();
    }).fail(function (data) {
        debugLog('failed /me');
        debugLog(data);

        if (isDoLogin) {
          doLogin(true);
        }
    });
  }

  /**
   * ログイン済みのアカウントのニュースフィード画面を表示する
   */
  function showNewsFeed() {
    debugLog('showNewsFeed start... ');
    
    var url = URLS.TOP;
    var wnd = window.open(url, '_blank', 'location=yes');

    wnd.addEventListener('loaderror', function (evt) {
        debugLog('window loaderror');
        debugLog(evt);
    });
    
    wnd.addEventListener('loadstart', function (evt) {
        debugLog('window loadstart');
        debugLog(evt);

        if (evt.url.indexOf('logout.php') !== -1) {
          // ログアウトされた
          clearAccessTokenInner();
          wnd.close();
        }
    });
  }
  
  /**
   * 空文字列チェック
   */
  function isEmpty(str) {
    return (typeof str !== 'string') || (str.length === 0);
  }
  function isNotEmpty(str) {
    return !isEmpty(str);
  }
  
  /**
   * URLクエリパラメータを分解して取得
   */
  function appendURLParam(url, key, value) {
    if (isEmpty(url) || isEmpty(key)) {
      return url;
    }
    
    if (url.indexOf('?') < 0) {
      url += '?';
    }
    else if (url.slice(-1) != '&') {
      url += '&';
    }
    
    return url + key + '=' + encodeURIComponent(value);
  }
  
  /**
   * デバッグログ出力
   */
  function debugLog(msg) {
    if (options.debug === true && console && typeof console.log === 'function') {
      console.log(msg);
    }
  }

  // ===== 以下、Public 関数群 =====
  return {
    /**
     * 初期化
     * @param {Object.<string, Object>} config 設定
     */
    init: function(config) {
      // Facebook OAuth接続のため、optionsには以下のキー/値を設定してください
      // {
      //   redirectUrlWhenLogin: '<ログイン後のリダイレクトURL>',
      //   redirectUrlWhenFeed: '<フィード送信後のリダイレクトURL>',
      //   clientId: '<Facebookアプリ クライアントID>',
      //   secretKey: '<Facebookアプリ シークレットキー>',
      // }
      options = (typeof config === 'object') ? config : {};
      
      // 取得済みアクセスキーの読込
      var accessKey = loadAccessKey();
      debugLog('stored access key');
      debugLog(accessKey);
      options.accessToken = accessKey.accessToken;
      options.userId = accessKey.userId;
    },
    
    /**
     * アクセストークンの所有確認
     * @return {Boolean} アクセストークンを保持していればtrue, それ以外ならfalse
     */
    hasAccessToken: function() {
      return hasAccessTokenInner();
    },
    
    /**
     * アクセストークンの破棄
     */
    clearAccessToken: function() {
      return clearAccessTokenInner();
    },
    
    /**
     * Facebook ログイン処理
     */
    login: function(callback) {
      doLogin(true,callback);
    },
    
    /**
     * Facebook ログアウト処理
     */
    logout: function() {
      doLogout();
    },

    /**
     * Facebook の Feed Dialog を利用して記事をシェアする
     * @param {string} linkUrl シェアする記事の URL
     * @param {string} name 記事のタイトル
     * @param {string} caption キャプション文字列
     */
    feed: function(linkUrl, name, caption) {
      doFeed(linkUrl, name, caption);
    }
  };
}();
